use crate::drawableVec::DrawableVec;

use color_eyre::{eyre::Ok, Result};



pub async fn sort(list:&mut DrawableVec) -> Result<()> {
    
    let mut max = usize::MIN;
    for i in list.getListClone().into_iter().map(|x| format!("{:b}",x)){

        if max < i.len(){
            max = i.len();
        }
 
    }
    

    let mut stack:Vec<(usize, usize)> = Vec::new();
    let mut stack2:Vec<(usize,usize)> = Vec::new();
    stack.push((0, list.len()));
    for i in (0..(max)).rev(){

        while stack.len() > 0{
            let cur = stack.pop().unwrap();

            match radix(list, i, cur.0, cur.1).await?{
                Some((initial, switch, max)) => {
 
                    if initial < switch{
                        stack2.push((initial, switch));
                    }
                    if switch < max{
  
                        stack2.push((switch, max));
                    }
 
                },
                None => return Ok(())
            }

        }
        
        stack = stack2.clone(); 
        stack2.clear(); 
    }

    Ok(())
}

async fn radix(list:&mut DrawableVec, radix:usize, mut minBoundry:usize, mut maxBoundry:usize) -> Result<Option<(usize,usize,usize)>>{
    let initialMin = minBoundry.clone();
    let initialMax = maxBoundry.clone();



    loop{
        if maxBoundry == minBoundry{
            break
        }
        let currentBit = get_bit_at(list.get(minBoundry), radix);

        if currentBit{
            list.swap(minBoundry, maxBoundry-1).await?;
            maxBoundry -= 1;
        }else{
            list.swap(minBoundry, minBoundry).await?;

            minBoundry += 1;
        }
    }


    
    Ok(Some((initialMin, minBoundry, initialMax)))
}
fn get_bit_at(input: u32, n: usize) -> bool {

    if format!("{:b}", input).len() <= n{
        false
    }else{
        input & (1 << n) != 0
    }


}

/*
#[cfg(test)]
#[allow(non_snake_case)]
mod tests {
  use crate::GuiHookVec::NonGuiVec;

use super::*;
  

  macro_rules! aw {
    ($e:expr) => {
        tokio_test::block_on($e)
    };
  }

  #[test]
  fn radixsort_correct() {
    let mut list:NonGuiVec = aw!(SortingList::new(1000,0.0));
    aw!(radixSort(&mut list));
    assert_eq!( list.isSorted(), true);
  }

  #[test]
  fn get_bit_at_test(){
    let num = 0b0001;
    let othernum = 2;
    assert_eq!(get_bit_at(num, 0), true);
    assert_eq!(get_bit_at(num, 1), false);
    assert_eq!(get_bit_at(othernum, 1), true);
    assert_eq!(get_bit_at(othernum, 2), false);
    assert_eq!(get_bit_at(othernum, 3), false);
  }
}



*/
